<template>
  <div class="mod-config">
    <basic-container>
      <ttable
        :vLoading="dataListLoading"
        :table="table"
        @handleSelectionChange="handleSelectionChange"
        :uploadInfo="uploadInfo"
        :prelang="prelang"
        :options="options"
        :ajaxFun="ajaxFun"
        ref="operationTaskTable"
      >
        <template #toolbar>
          <el-dropdown
            v-if="permissions.operationtask_state_change"
            @command="handleCommand"
          >
            <el-button>
              状态改变<i class="el-icon-arrow-down el-icon--right"></i>
            </el-button>
            <el-dropdown-menu slot="dropdown">
              <el-dropdown-item
                :command="item.command"
                v-for="item in stateTagArr"
                :disabled="item.disabled"
                >{{ item.label }}</el-dropdown-item
              >
            </el-dropdown-menu>
          </el-dropdown>

          <el-button
            v-if="permissions.operationtask_task_merge"
            style="margin-left:10px"
            @click="combineOrder"
          >
            合并工单
          </el-button>
          <el-button style="margin-left:10px" @click="openTaskOrder">
            优先级
          </el-button>
          <el-button
          type="primary"
            v-if="permissions.operationtask_pallet_transport"
            style="margin-left:10px"
            @click="openTaskMaterial"
          >
            打印模具领用清单
          </el-button>
          <el-button
          type="primary"
            v-if="permissions.operationtask_pallet_transport"
            style="margin-left:10px"
            @click="openTaskMaterial"
          >
            打印物料领用清单
          </el-button>
        </template>
      </ttable>
      <!-- 弹窗, 新增 / 修改 -->
      <table-form
        v-if="addOrUpdateVisible"
        ref="addOrUpdate"
        @refreshDataList="getData"
      ></table-form>
      <mergeTask v-if="mergeTaskVisible" ref="mergeTask"></mergeTask>

      <taskMaterialDialog
        :currshowlist.sync="showTaskMaterial"
        :operationTasks="multipleSelection"
      />
      <taskOrderDialog
        :currshowlist.sync="showTaskOrder"
        :operationTasks="multipleSelection"
        @refreshDataList="getData"
      />
    </basic-container>
  </div>
</template>
<script>
import { fetchList, delObj, getOptaskCode } from '@/api/plan/operationtask'
import ttable from '@/views/common/ztt-table.vue'
import TableForm from './operationtask-form'
import {
  changeState,
  combineOperationTask
} from '../../../api/plan/operationtask'
import { dir } from '@/api/quality/template'
import mergeTask from './merge-task.vue'
import taskMaterialDialog from './task-material'
import taskOrderDialog from './task-order'
import { mapGetters } from 'vuex'
import { remote } from '@/api/admin/dict'

export default {
  data() {
    return {
      stateTagArr: [
        {
          label: '标记等待',
          command: 'UNSUBMIT',
          disabled: false,
          permitArr: ['07unsubmit']
        },
        {
          label: '标记进行中',
          command: 'START',
          disabled: false,
          permitArr: ['01pending', '03interrupted', '04completed']
        },
        {
          label: '标记已暂停',
          command: 'INTERRUPT',
          disabled: false,
          permitArr: ['02inProgress']
        },
        {
          label: '标记已完成',
          command: 'COMPLETE',
          disabled: false,
          permitArr: ['02inProgress']
        },
        {
          label: '标记已取消',
          command: 'CANCEL',
          disabled: false,
          permitArr: ['01pending', '02inProgress', '03interrupted']
        }
      ],
      manufacturingVisible: false,
      ajaxFun: fetchList,
      dataListLoading: false,
      multipleSelection: [],
      isShowQuery: false,
      uploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      dateTimeFilters: {},
      prelang: 'operationtask',
      options: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: true, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮H
        isSearch: true, // 高级查询按钮
        getSummaries: this.getSummaries,
        defaultOrderBy: { column: 'createTime', direction: 'desc' }
      },
      table: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          {
            minWidth: '140',
            prop: 'optaskNo',
            label: '工单编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text',
            render: { fun: this.addOrUpdateHandle }
          },
          {
            minWidth: '120',
            prop: 'state',
            label: '状态',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            optList: () => {
              return this.getStateOptionList()
            },
            formatter: (row, column, cellValue) => {
              var formatVal
              if (cellValue == '07unsubmit') {
                formatVal = '待审核'
              } else if (cellValue == '01pending') {
                formatVal = '等待'
              } else if (cellValue == '02inProgress') {
                formatVal = '进行中'
              } else if (cellValue == '03interrupted') {
                formatVal = '已暂停'
              } else if (cellValue == '04completed') {
                formatVal = '已完成'
              } else if (cellValue == '05canceled') {
                formatVal = '已取消'
              } else if (cellValue == '06combined') {
                formatVal = '已合并'
              } else {
                formatVal = ''
              }
              return formatVal
            }
          },
          {
            minWidth: '140',
            prop: 'customerOrderNo',
            label: '销售订单',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'customerName',
            label: '客户名称',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'mpsNo',
            label: '主计划号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'moNo',
            label: '车间订单编号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'combineState',
            label: '工单类型',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            optList: () => {
              return this.combineStateOptions
            },
            render: {
              fun: this.searchMergeOrder,
              rowValue: (cellValue) => {
                this.combineStateOptions.forEach((obj) => {
                  if (obj.value === String(cellValue)) {
                    cellValue = obj.label
                  }
                })
                return cellValue
              }
            }
          },
          {
            minWidth: '140',
            prop: 'operationTaskType',
            label: '类型',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.formatWorkshopTypeDbType,
            optList: () => {
              return this.workshopTypeOptions
            }
          },
          {
            minWidth: '120',
            prop: 'manufactureAttr',
            label: '制造属性',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'select',
            formatter: this.getManufactureAttr,
            optList: () => {
              return this.manufactureAttrs
            }
          },
          {
            minWidth: '140',
            prop: 'partName',
            label: '零件',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'workCenter',
            label: '工作中心',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'workstationName',
            label: '工作站',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'operationName',
            label: '工序',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'priority',
            label: '优先级',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '140',
            prop: 'hourQuota',
            label: '生产工时定额',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'plannedQuantity',
            label: '计划数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '20',
            prop: 'unit',
            label: '单位',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'splannedQuantity',
            label: '计划数量2',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '20',
            prop: 'sunit',
            label: '单位2',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'completedQuantity',
            label: '完成数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // {
          //   minWidth: '120',
          //   prop: 'discNum',
          //   label: '盘数',
          //   sort: true,
          //   isTrue: true,
          //   isSearch: true,
          //   searchInfoType: 'text'
          // },
          // {
          //   minWidth: '120',
          //   prop: 'reelSpec',
          //   label: '载具规格',
          //   sort: true,
          //   isTrue: true,
          //   isSearch: true,
          //   searchInfoType: 'text'
          // },
          {
            width: '130',
            prop: 'ratio',
            label: '完成进度(%)',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'remark',
            label: '备注',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '120',
            prop: 'createTime',
            label: '创建时间',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'plannedStartDate',
            label: '计划开始时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'plannedFinishDate',
            label: '计划完成时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'actualStartDate',
            label: '实际开始时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'actualFinishDate',
            label: '实际完成时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange',
            formatter: this.formatDutyDate
          },
          {
            minWidth: '120',
            prop: 'remark',
            label: '备注',
            sort: false,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text'
          }
        ],
        toolbar: [],
        operator: [
          {
            text: '删除',
            icon: 'el-icon-delete',
            type: 'text',
            size: 'small',
            fun: this.deleteHandle
          }
        ],
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 100,
          minWidth: 100
        }
      },
      addOrUpdateVisible: false,
      stateOptionList: [
        {
          value: '07unsubmit',
          label: '待审核'
        },
        {
          value: '01pending',
          label: '等待'
        },
        {
          value: '02inProgress',
          label: '进行中'
        },
        {
          value: '03interrupted',
          label: '已暂停'
        },
        {
          value: '04completed',
          label: '已完成'
        },
        {
          value: '05canceled',
          label: '已取消'
        },
        {
          value: '06combined',
          label: '已合并'
        }
      ],
      combineStateOptions: [],
      mergeTaskVisible: false,
      workshopTypeOptions: [],
      showTaskMaterial: false,
      manufactureAttrs: [],
      showTaskOrder: false
    }
  },
  components: {
    ttable,
    TableForm,
    mergeTask,
    taskMaterialDialog,
    taskOrderDialog
  },
  created() {
    this.getManufactureAttrs()
  },
  mounted() {
    this.initDirTestType('combineState_type')
    this.getWorkshopTypeOptions('workshop_type')
  },
  computed: {
    ...mapGetters(['permissions'])
  },
  activated() {
    this.getData()
  },
  methods: {
    getManufactureAttr(row, column, cellValue) {
      this.manufactureAttrs.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    getManufactureAttrs() {
      remote('manufacture_attr_type').then((response) => {
        if (response.data.code === 0) {
          this.manufactureAttrs = response.data.data
        } else {
          this.manufactureAttrs = []
        }
      })
    },
    getSummaries(param) {
      const { columns, data } = param
      const sums = []
      columns.forEach((column, index) => {
        if (index === 0) {
          sums[index] = '合计'
          return
        }
        const values = data.map((item) => Number(item[column.property]))
        if (
          column.property == 'plannedQuantity' ||
          column.property == 'completedQuantity'
        ) {
          if (!values.every((value) => isNaN(value))) {
            sums[index] = values.reduce((prev, curr) => {
              const value = Number(curr)
              if (!isNaN(value)) {
                return Number((prev + curr).toFixed(2))
              } else {
                return prev
              }
            }, 0)
          } else {
            sums[index] = ''
          }
        } else {
          sums[index] = ''
        }
      })
      return sums
    },
    // 获取数据列表
    getData() {
      this.$refs.operationTaskTable.getDataList()
    },
    // 新增 / 修改
    addOrUpdateHandle(row) {
      this.addOrUpdateVisible = true
      this.$nextTick(() => {
        this.$refs.addOrUpdate.init(row == null ? null : row.id)
      })
    },
    formatDutyDate(row, column, cellValue) {
      if (cellValue) {
        const dutyDateList = /\d{4}-\d{1,2}-\d{1,2}/g.exec(cellValue)
        if (dutyDateList && dutyDateList.length > 0) {
          return dutyDateList[0]
        }
      }
      return ''
    },
    // 标记状态
    handleCommand(event) {
      let eventStr = ''
      const eventStateStr = ''
      switch (event) {
        case 'UNSUBMIT':
          eventStr = '等待'
          break
        case 'START':
          eventStr = '进行中'
          break
        case 'INTERRUPT':
          eventStr = '暂停'
          break
        case 'COMPLETE':
          eventStr = '完成'
          break
        case 'CANCEL':
          eventStr = '取消'
          break
        default:
          break
      }
      if (this.multipleSelection && this.multipleSelection.length > 0) {
        // 判断一下选择的状态是否一致
        changeState(
          this.multipleSelection.map((item) => {
            return item.id
          }),
          event
        ).then((data) => {
          this.$message.success('标记' + eventStr + '成功')
          this.getData()
        })
      } else {
        this.$message.error('请选择工单')
      }
    },
    // 删除
    deleteHandle(row) {
      if (row.state == '07unsubmit') {
        this.$confirm('是否确认删除ID为' + row.id, '提示', {
          confirmButtonText: '确定',
          cancelButtonText: '取消',
          closeOnClickModal: false,
          type: 'warning'
        })
          .then(function() {
            return delObj(row.id)
          })
          .then((response) => {
            var data = response.data
            if (data.code == 0) {
              this.$message.success('删除成功')
            } else {
              this.$message.error('删除失败')
            }
            this.getData()
          })
      } else {
        this.$message.error('工单在该状态下，不可进行删除')
      }
    },
    handleSelectionChange(val) {
      // 根据状态，禁用表头按钮
      // 筛选出选中记录的状态
      var stateArr = val.map(function(value, index) {
        return value.state
      })
      // 选中状态数组元素去重
      var uniqueStateArr = []
      for (var i = 0; i < stateArr.length; i++) {
        if (uniqueStateArr.indexOf(stateArr[i]) == -1) {
          uniqueStateArr.push(stateArr[i])
        }
      }
      // 循环自定义按钮，判断每个按钮的permitArr是否完全包含选中状态，若完全包含，则按钮亮，否则按钮灰
      for (var i = 0; i < this.stateTagArr.length; i++) {
        if (
          uniqueStateArr.every((val) =>
            this.stateTagArr[i].permitArr.length <= 0
              ? true
              : this.stateTagArr[i].permitArr.includes(val)
          )
        ) {
          this.stateTagArr[i].disabled = false
        } else {
          this.stateTagArr[i].disabled = true
        }
      }
      this.multipleSelection = val
    },
    getStateOptionList() {
      return this.stateOptionList
    },
    // 合并工单
    combineOrder() {
      if (this.multipleSelection.length > 0) {
        const ids = this.multipleSelection.map((item) => {
          return item.id
        })
        combineOperationTask(ids).then((res) => {
          this.$message.success('合并成功')
          this.getData()
        })
      } else {
        this.$message.error('请选择工单')
      }
    },
    // 数据字典传值
    initDirTestType(index) {
      dir(index).then((res) => {
        this.combineStateOptions = res.data.data
      })
    },
    searchMergeOrder(val) {
      this.mergeTaskVisible = true
      this.$nextTick(() => {
        this.$refs.mergeTask.init(val.id)
      })
    },
    getWorkshopTypeOptions(index) {
      dir(index).then((res) => {
        this.workshopTypeOptions = res.data.data
      })
    },
    // 字典格式化
    formatWorkshopTypeDbType(row, column, cellValue) {
      this.workshopTypeOptions.forEach((obj) => {
        if (obj.value === cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    // 打开需求零件弹框
    openTaskMaterial() {
      if (this.multipleSelection.length > 0) {
        // 校验工单是否已生成货盘运输任务
        const ids = []
        this.multipleSelection.forEach((item) => {
          ids.push(item.id)
        })
        getOptaskCode(ids).then((response) => {
          const resData = response.data
          if (resData.code === 0) {
            this.showTaskMaterial = true
          } else {
            this.$message.error(resData.msg)
          }
        })
      } else {
        this.$message.error('请选择工单')
      }
    },
    // 打开工单排序页面
    openTaskOrder() {
      if (this.multipleSelection.length > 1) {
        let flag = true
        this.multipleSelection.forEach((item) => {
          if (!(item.state == '07unsubmit' || item.state == '01pending')) {
            flag = false
          }
        })
        if (flag) {
          this.showTaskOrder = true
        } else {
          this.$message.error('只有【状态】为待审核、等待的工单，才能进行排序')
        }
      } else {
        this.$message.error('请选择至少两条工单，进行排序')
      }
    },
    formatRatio(row, column, cellValue) {
      if (cellValue) {
        return cellValue + '%'
      }
      return ''
    }
  }
}
</script>
